@@ -1,8 +1,8 @@
 
 package Coro::ProcessPool;
-$Coro::ProcessPool::VERSION = '0.26_003'; # TRIAL
+$Coro::ProcessPool::VERSION = '0.26_004'; # TRIAL
 
-$Coro::ProcessPool::VERSION = '0.26003';# ABSTRACT: An asynchronous process pool
+$Coro::ProcessPool::VERSION = '0.26004';# ABSTRACT: An asynchronous process pool
 
 use Moo;
 use Types::Standard qw(-types);
@@ -21,28 +21,24 @@ if ($^O eq 'MSWin32') {
   die 'MSWin32 is not supported';
 }
 
-
 has max_procs => (
   is      => 'ro',
   isa     => Int,
   default => sub { Coro::ProcessPool::Util::cpu_count() },
 );
 
-
 has max_reqs => (
   is      => 'ro',
   isa     => Int,
   default => sub { 0 },
 );
 
-
 has include => (
   is      => 'ro',
   isa     => ArrayRef[Str],
   default => sub { [] },
 );
 
-
 has procs_lock => (
   is  => 'lazy',
   isa => InstanceOf['Coro::Semaphore'],
@@ -53,28 +49,24 @@ sub _build_procs_lock {
   return Coro::Semaphore->new($self->max_procs);
 }
 
-
 has num_procs => (
   is      => 'rw',
   isa     => Int,
   default => sub { 0 },
 );
 
-
 has procs => (
   is      => 'ro',
   isa     => ArrayRef[InstanceOf['Coro::ProcessPool::Process']],
   default => sub { [] },
 );
 
-
 has all_procs => (
   is      => 'ro',
   isa     => HashRef[InstanceOf['Coro::ProcessPool::Process']],
   default => sub { {} },
 );
 
-
 has is_running => (
   is      => 'rw',
   isa     => Bool,
@@ -107,9 +99,14 @@ sub start_proc {
 
 sub kill_proc {
   my ($self, $proc) = @_;
-  $proc->stop;
-  --$self->{num_procs};
+
   delete $self->{all_procs}{$proc->pid};
+  --$self->{num_procs};
+
+  async_pool {
+    $proc->stop;
+    $proc->join;
+  };
 }
 
 sub checkin_proc {
@@ -120,13 +117,7 @@ sub checkin_proc {
     return;
   }
 
-  if (!$proc->alive) {
-    my $pid = $proc->pid;
-    --$self->{num_procs};
-    delete $self->{all_procs}{$pid};
-    unshift @{$self->procs}, $self->start_proc;
-  }
-  elsif ($self->max_reqs && $proc->{counter} >= $self->max_reqs) {
+  if (!$proc->alive || ($self->max_reqs && $proc->{counter} >= $self->max_reqs)) {
     $self->kill_proc($proc);
     unshift @{$self->procs}, $self->start_proc;
   }
@@ -151,22 +142,18 @@ sub checkout_proc {
   return $proc;
 }
 
-
 sub capacity {
   my $self = shift;
   return scalar(@{$self->procs});
 }
 
-
 sub shutdown {
   my $self = shift;
 
   $self->is_running(0);
 
-  foreach (values %{$self->{all_procs}}) {
-    $_->stop;
-    $_->join;
-  }
+  $self->kill_proc($_)
+    foreach values %{$self->{all_procs}};
 
   $self->{procs}      = [];
   $self->{all_proces} = {};
@@ -176,7 +163,6 @@ sub shutdown {
   return;
 }
 
-
 sub process {
   my ($self, $f, $args) = @_;
   my $guard = $self->procs_lock->guard;
@@ -185,17 +171,15 @@ sub process {
   $proc->send($f, $args)->recv;
 }
 
-
 sub map {
   my ($self, $f, @args) = @_;
   my @deferred = map { $self->defer($f, [$_]) } @args;
   return map { $_->() } @deferred;
 }
 
-
 sub defer {
   my $self = shift;
-  my $cv   = AnyEvent->condvar;
+  my $cv = AnyEvent->condvar;
 
   async_pool {
     my ($self, $cv, @args) = @_;
@@ -204,16 +188,14 @@ sub defer {
     $cv->send($result);
   } $self, $cv, @_;
 
-  return sub { $cv->recv };
+  return sub{ $cv->recv };
 }
 
-
 sub pipeline {
   my $self = shift;
   return Coro::ProcessPool::Pipeline->new(pool => $self, @_);
 }
 
-
 1;
 
 __END__
@@ -228,7 +210,7 @@ Coro::ProcessPool - An asynchronous process pool
 
 =head1 VERSION
 
-version 0.26_003
+version 0.26_004
 
 =head1 SYNOPSIS
 
