@@ -1,9 +1,12 @@
 package Coro::ProcessPool::Process;
-$Coro::ProcessPool::Process::VERSION = '0.26_003'; # TRIAL
+# ABSTRACT Manager for worker subprocess
+$Coro::ProcessPool::Process::VERSION = '0.26_004'; # TRIAL
 
-$Coro::ProcessPool::Process::VERSION = '0.26003';use strict;
+$Coro::ProcessPool::Process::VERSION = '0.26004';use strict;
 use warnings;
+use Carp;
 use Coro;
+use Coro::Countdown;
 use Data::UUID;
 use Coro::AnyEvent;
 use POSIX qw(:sys_wait_h);
@@ -22,8 +25,8 @@ sub worker {
   my $args  = get_args(@$inc);
   my $exec  = "$cmd $args";
 
-  my ($parent_in, $child_out) = portable_pipe;
-  my ($child_in, $parent_out) = portable_pipe;
+  my ($child_in, $parent_out)  = portable_pipe;
+  my ($parent_in, $child_out)  = portable_pipe;
 
   my $proc = bless {
     pid     => undef,
@@ -34,6 +37,7 @@ sub worker {
     stopped => undef,
     started => AE::cv,
     counter => 0,
+    pending => Coro::Countdown->new,
   }, 'Coro::ProcessPool::Process';
 
   $proc->{stopped} = run_cmd $exec, (
@@ -42,15 +46,12 @@ sub worker {
     '>'  => $child_out,
     '<'  => $child_in,
     '2>' => sub {
-      my $err = shift;
-      return unless defined $err;
-      warn "worker error: $err\n";
-    }
+      my $err = shift or return;
+      warn "[worker pid:$proc->{pid}] $err\n";
+    },
   );
 
   $proc->{stopped}->cb(sub {
-    my $cv = shift;
-    $cv->recv;
     $proc->{in}->close;
     $proc->{out}->close;
   });
@@ -75,6 +76,8 @@ sub worker {
         }
 
         delete $proc->{inbox}{$id};
+        $proc->{pending}->down;
+
       } else {
         warn "Unexpected message received: $id";
       }
@@ -84,7 +87,10 @@ sub worker {
   return $proc;
 }
 
-sub pid { $_[0]->{pid} }
+sub pid {
+  my $proc = shift;
+  return $proc->{pid};
+}
 
 sub await {
   my $proc = shift;
@@ -93,6 +99,7 @@ sub await {
 
 sub join {
   my $proc = shift;
+  $proc->{pending}->join;
   $proc->{stopped}->recv;
 }
 
@@ -106,28 +113,31 @@ sub alive {
 
 sub stop {
   my $proc = shift;
-  $proc->send('self-terminate') if $proc->alive;;
+  if ($proc->alive) {
+    $proc->{out}->print(encode('', 'self-terminate', []) . $EOL);
+  }
 }
 
 sub kill {
   my $proc = shift;
-  kill('KILL', $proc->{pid}) if $proc->alive;
+  if ($proc->alive) {
+    kill('KILL', $proc->{pid});
+  }
 }
 
 sub send {
   my ($proc, $f, $args) = @_;
+  croak 'subprocess is not running' unless $proc->alive;
 
   # Add a watcher to the inbox for this task
   my $id = Data::UUID->new->create_str;
   $proc->{inbox}{$id} = AE::cv;
 
   # Send the task to the worker
-  async_pool {
-    my ($proc, $id, $f, $args) = @_;
-    $proc->{out}->print(encode($id, $f, $args || []) . $EOL);
-  } $proc, $id, $f, $args;
+  $proc->{out}->print(encode($id, $f, $args || []) . $EOL);
 
   ++$proc->{counter};
+  $proc->{pending}->up;
 
   return $proc->{inbox}{$id};
 }
@@ -146,7 +156,7 @@ Coro::ProcessPool::Process
 
 =head1 VERSION
 
-version 0.26_003
+version 0.26_004
 
 =head1 AUTHOR
 
