@@ -1,16 +1,16 @@
 package Coro::ProcessPool::Util;
 # ABSTRACT: Utility functions and constants used by Coro::ProcessPool
-$Coro::ProcessPool::Util::VERSION = '0.26_003'; # TRIAL
+$Coro::ProcessPool::Util::VERSION = '0.26_004'; # TRIAL
 
-$Coro::ProcessPool::Util::VERSION = '0.26003';use strict;
+$Coro::ProcessPool::Util::VERSION = '0.26004';use strict;
 use warnings;
 use Carp;
 use Config;
 use Const::Fast;
 use Data::Dump::Streamer;
-use MIME::Base64  qw(encode_base64 decode_base64);
+use MIME::Base64 qw(encode_base64 decode_base64);
 use String::Escape  qw(backslash);
-use Sereal::Encoder qw(sereal_encode_with_object SRL_SNAPPY);
+use Sereal::Encoder qw(sereal_encode_with_object);
 use Sereal::Decoder qw(sereal_decode_with_object);
 
 use parent 'Exporter';
@@ -25,7 +25,6 @@ our @EXPORT_OK = qw(
 );
 
 const our $EOL => "\n";
-const our $DEFAULT_READ_TIMEOUT => 0.1;
 
 my $ENCODER = Sereal::Encoder->new();
 my $DECODER = Sereal::Decoder->new();
@@ -73,7 +72,7 @@ sub get_command_path {
 
 sub get_args {
   my @inc = map { sprintf('-I%s', backslash($_)) } @_, @INC;
-  my $cmd = q|-MCoro::ProcessPool::Worker -e 'Coro::ProcessPool::Worker->new->run'|;
+  my $cmd = q|-MCoro::ProcessPool::Worker -e 'Coro::ProcessPool::Worker::run()'|;
   return join ' ', @inc, $cmd;
 }
 
@@ -166,7 +165,7 @@ Coro::ProcessPool::Util - Utility functions and constants used by Coro::ProcessP
 
 =head1 VERSION
 
-version 0.26_003
+version 0.26_004
 
 =head1 AUTHOR
 
